@@ -3,9 +3,10 @@ Login and signup views.
 Handles create new user accounts and authenticating users.
 """
 from bottle import request, response, template, local, redirect, default_app, get, post
-
+import requests
+import os
 from codalab.lib import crypt_util, spec_util
-from codalab.lib.server_util import redirect_with_query
+from codalab.lib.server_util import redirect_with_query, safe_uri
 from codalab.lib.spec_util import NAME_REGEX
 from codalab.common import UsageError
 from codalab.objects.user import User
@@ -29,7 +30,7 @@ def send_verification_key(username, email, key):
 @get('/account/logout', name='logout', skip=UserVerifiedPlugin)
 def do_logout():
     LoginCookie.clear()
-    redirect_uri = request.query.get('redirect_uri')
+    redirect_uri = safe_uri(request.query.get('redirect_uri'))
     return redirect(redirect_uri)
 
 
@@ -55,7 +56,7 @@ def do_login():
 
     # Redirect client to next page
     if success_uri:
-        return redirect(success_uri)
+        return redirect(safe_uri(success_uri))
     else:
         return redirect('/')
 
@@ -75,8 +76,28 @@ def do_signup():
     last_name = request.forms.get('last_name')
     password = request.forms.get('password')
     affiliation = request.forms.get('affiliation')
+    token = request.forms.get('token')
 
     errors = []
+
+    if not token:
+        errors.append('Google reCAPTCHA token is missing.')
+    else:
+        url = 'https://www.google.com/recaptcha/api/siteverify'
+        data = {
+            'secret': os.environ['CODALAB_RECAPTCHA_SECRET_KEY'],
+            'response': token,
+        }
+        res = requests.post(url, data)
+
+        try:
+            data = res.json()
+            if not data.get('success'):
+                errors.append('Google reCAPTCHA failed.')
+
+        except UsageError as e:
+            errors.append(str(e))
+
     if request.user.is_authenticated:
         errors.append(
             "You are already logged in as %s, please log out before "
