@@ -191,6 +191,9 @@ class BundleCheckinState(object):
         remote,  # type: str
         exitcode,  # type: Optional[str]
         failure_message,  # type: Optional[str]
+        bundle_profile_stats,  # type: dict
+        cpu_usage,  # type: float
+        memory_usage,  # type: int
     ):
         self.uuid = uuid
         self.run_status = run_status
@@ -203,6 +206,9 @@ class BundleCheckinState(object):
         self.remote = remote
         self.exitcode = exitcode
         self.failure_message = failure_message
+        self.cpu_usage = cpu_usage
+        self.memory_usage = memory_usage
+        self.bundle_profile_stats = bundle_profile_stats
 
     @classmethod
     def from_dict(cls, dct):
@@ -218,6 +224,9 @@ class BundleCheckinState(object):
             remote=dct['remote'],
             exitcode=dct['exitcode'],
             failure_message=dct['failure_message'],
+            cpu_usage=dct.get('cpu_usage'),
+            memory_usage=dct.get('memory_usage'),
+            bundle_profile_stats=dct.get('bundle_profile_stats'),
         )
 
     @property
