@@ -70,6 +70,12 @@ def main():
         default=10 * 60,
         type=int,
     )
+    parser.add_argument(
+        '--worker-download-dependencies-max-retries',
+        type=int,
+        default=3,
+        help='The number of times a CodaLab worker will retry downloading dependencies after a failure (defaults to 3).',
+    )
     parser.add_argument(
         '--worker-checkin-frequency-seconds',
         type=int,
